"""
This is the configuration file for AutoGAMESS.

Houses various variables/functions used commonly by AutoGAMESS.
"""
import os
import sys
import numpy  as np
import pandas as pd
import pkg_resources
import basis_set_exchange as bse

version = (' AutoGAMESS Version ' +
            str(pkg_resources.require("autogamess")[0].version) )
author  = '     by Brian C. Ferrari'
agv     = 'AGv' + version.split(' ')[-1].replace('.', '-')

#Error messages
error_head   = "\n*****uh oh spaghettios*****\n"
error_tail   = "\n*****Ponder this, then return to me*****\n"

#Basic input file parameters for optimization calculation
basic_params = (' $CONTRL SCFTYP=RHF MULT=1 NPRINT=0 COORD=UNIQUE\n',
                ' RUNTYP=OPTIMIZE ICUT=12 ITOL=25 theory\n',
                ' MAXIT=200 QMTTOL=1E-7 ICHARG=0 ISPHER=1 $END\n',
                ' $SYSTEM MWORDS=800 MEMDDI=800 $END\n',
                ' $STATPT OPTTOL=1E-6 NSTEP=200 $END\n',
                ' $SCF DIRSCF=.TRUE. FDIFF=.FALSE. CONV=1d-7 $END\n',
                ' $DFT JANS=2 $END\n',
                ' $BASIS GBASIS=basis $END\n',
                ' $DATA\n')

#Basic input file parameters for composite method calculation
comp_basic_params = (' $CONTRL SCFTYP=RHF MULT=1 NPRINT=0 COORD=UNIQUE\n',
                     ' RUNTYP=COMP ICUT=12 ITOL=25 MAXIT=200\n',
                     ' QMTTOL=1E-7 ICHARG=0 ISPHER=1 NOSYM=1 $END\n',
                     ' $COMP theory $END\n',
                     ' $SYSTEM MWORDS=800 MEMDDI=800 $END\n',
                     ' $STATPT OPTTOL=1E-6 NSTEP=200 $END\n',
                     ' $SCF DIRSCF=.TRUE. FDIFF=.FALSE. CONV=1d-7\n',
                     ' DAMP=.T. $END\n',
                     ' $DATA\n')

#Basic functions used throughout code-----------------------------------
def check_if_exists(filename, *args):
    for arg in args:
        if arg == -1:
            msg = "Something went wrong, check your log file\n" + filename
            print(error_head + msg + error_tail)
            return True
    return False

def check_if_in(filename, *args, look_here):
    for arg in args:
        if arg not in look_here:
            msg = arg + "\n Not Found in " + filename
            sys.exit(error_head + msg + error_tail)
    return

def ctr_f(find_this, look_here):
    for line in look_here:
        if find_this in line:
            return look_here.index(line)
    return -1

def ctr_f_all(find_this, look_here):
    r = []
    for line in look_here:
        if "TERMINATED -ABNORMALLY-" in line:
            r.append("N/A")
            r.append("N/A")
        if find_this in line:
            r.append(line.split(':')[1])

    return r

def ctr_f_last(find_this, look_here):
    look_here.reverse()
    for line in look_here:
        if find_this in line:
            look_here.reverse()
            return look_here.index(line)
    return -1

def angle_between(v1, v2):
    v1hat = v1 / np.linalg.norm(v1)
    v2hat = v2 / np.linalg.norm(v2)
    return np.arccos(np.clip(np.dot(v1hat, v2hat), -1.0, 1.0))

def find_bond_angle(o,i,j):
    v1 = np.array(i) - np.array(o)
    v2 = np.array(j) - np.array(o)
    return angle_between(v1,v2)

def make_xzy(xyzlist):
    r = (float(xyzlist[0]),
         float(xyzlist[1]),
         float(xyzlist[2]))
    return r

def flatten(array):
    r = []
    for i in array:
        r += i
    return r

def check_data_type(data):
    #Check if data is in CSV format
    if '.csv' in data:
        df = pd.read_csv(data)

    #Terminates function with error message if data format not acceptable
    elif type(data) is not type(pd.DataFrame()):
        print(error_head)
        print("Your data is not in CSV or Pandas DataFrame Format")
        print(error_tail)
        sys.exit()

    #If data already DataFrame format, changes its name to df
    else:
        df = data

    return df

def read_file(file):
    f=open(file, 'r')
    r=f.readlines()
    f.close()
    return r

def get_gamess_input(command, doc):
    i = ctr_f(command, doc)
    j = ctr_f(command, doc[i].split())
    return doc[i].split()[j].split('=')[1]

def gaussian(x, mu, sig, amp):
    return  np.exp(-(x-mu)**2 / (2.*sig**2)) * amp

def lorentzian(x, mu, sig, amp):
    return ( 1/( 1 + ( ((x-mu)**2)/(sig**2)) ) ) * amp

def scaling_factor_scott(theo, expt):
    top    = 0
    top2   = 0
    bottom = 0
    n      = range(len(theo))
    for i in n:
        top    += theo[i]*expt[i]
        bottom += theo[i]**2

    c  = round(top/bottom, 4)
    for i in n:
        top2 += (c*theo[i] - expt[i])**2

    rms = round(np.sqrt(top2/len(theo)), 2)

    return c, rms

def get_elements(molecule):
    atoms = []

    for i in range(len(molecule)):
        x = molecule[i]

        if x.isupper() and x.isalpha():
            if i != len(molecule)-1:
                y = molecule[i+1]

            if y.islower():
                atoms.append(x+y)
                y = ''
            else:
                atoms.append(x)

    return atoms

def conversion_factor(old, new):
    daa = 'Debye^2 angstrom^-2 amu^-1'
    kmm = 'km mol^-1'
    cmm = 'cm molecule^-1'

    if (old == daa) and (new == kmm):
        return 42.2547
    if (old == daa) and (new == cmm):
        return 7.017e-18
    if (old == kmm) and (new == daa):
        return (1/42.2547)
    if (old == kmm) and (new == cmm):
        return 1.661e-19
    if (old == cmm) and (new == daa):
        return (1/7.017e-18)
    if (old == cmm) and (new == kmm):
        return (1/1.661e-19)

    print(error_head)
    print("Your old or new units choice did not match any available options")
    print(error_tail)
    sys.exit()



from ..config import *
import matplotlib.pyplot as plt
from scipy.stats import ttest_ind, linregress

def make_scatter(data, savedir):
    """
    """
    #Define Sheet names
    opt = 'Optimization'
    hes = 'Hessian'
    ram = 'Raman'
    vsc = 'VSCF'

    #Define Data Column names/variables
    rt = 'Run Time'
    bs = 'Basis Set'
    cp = 'CPU Percentage'
    te = 'Theory'
    me = 'Method'
    fe = 'Freq'
    bl = 'Bond Length'
    ir = 'Infrared'
    ra = 'Raman'

    #Other predefined strings
    png      = '.png'
    xlsx     = '.xlsx'
    engine   = 'xlsxwriter'
    X        = 'X'

    #Predefine Markers and Colors
    markers  = ['X', '<', '>', 's', 'o', 'd', '*', '^', 'P', 'p']
    colors   = ['r','k','blue','darkorange', 'lime','olive', 'deepskyblue',
                'purple', 'gold']


    if xlsx in data:
        data = pd.read_excel(data,  index_col=0, sheet_name=None, header=6)

    for name, df in data.items():
        ref = df.columns[0]

        for col in df:
            dir_path = savedir + name + '/' + col.replace('/', '_') + '/'

            try:
                os.makedirs(dir_path)
            except:
                pass

            x   = df[ref]
            y   = df[col]
            l   = np.arange(min(x), max(x), (max(x) - min(x))/100)
            try:
                mask = ~np.isnan(x) & ~np.isnan(y)
                m, inter, r, p_val, sig = linregress(x[mask], y[mask])
                t,p = ttest_ind(x[mask], y[mask], equal_var=False)
                s   = 't = ' + str(round(t, 3)) + '\np = ' + str(round(p, 3)) +\
                      '\nr² = ' + str(round(r**2, 3))
            except:
                s   = ''

#--------------------Original Plot----------------------------------
            fig, ax = plt.subplots()
            plt.scatter(x, y, marker='x', color='red', label=col)
            plt.plot(l, l, color='black', linewidth=0.75, label=None)
            if s:
                txt = plt.text(0.05, 0.8, s, transform=ax.transAxes)
            plt.legend()
            plt.tight_layout()
            plt.savefig(dir_path + 'Linear.png')
            if 'Bond' in name:
                plt.close()
            else:
                plt.yscale('symlog')
                plt.xscale('symlog')
                plt.savefig(dir_path + 'LogLog.png')
                plt.close()
#--------------------Original Plot----------------------------------


#--------------------Difference Plot----------------------------------
            plt.scatter(x, y-x, marker='x', color='red', label=col)
            plt.plot(l, l-l, color='black', linewidth=0.75, label=None)
            plt.legend()
            plt.tight_layout()
            plt.savefig(dir_path + 'Linear--Dif.png')
            if 'Bond' in name:
                plt.close()
            else:
                plt.yscale('symlog')
                plt.xscale('symlog')
                plt.savefig(dir_path + 'LogLog--Dif.png')
                plt.close()
#--------------------Difference Plot----------------------------------


#--------------------Combination Plot----------------------------------
        i=0
        dir_path = savedir + name + '/'
        fig, ax = plt.subplots()
        for col in df:
            if col == ref:
                continue
            df.plot.scatter(ref, col, marker=markers[i], edgecolor=colors[i],
                            label=col, ax=ax, c='none')
            i += 1
        plt.plot(l, l, color='black', linewidth=0.75, label=None)
        plt.legend()
        plt.tight_layout()
        plt.savefig(dir_path + 'Linear--All.png')
        if 'Bond' in name:
            plt.close()
        else:
            plt.yscale('symlog')
            plt.xscale('symlog')
            plt.savefig(dir_path + 'LogLog--All.png')
            plt.close()
#--------------------Combination Plot----------------------------------


#--------------------Combination Diff Plot----------------------------------
        i=0
        dir_path = savedir + name + '/'
        fig, ax = plt.subplots()
        for col in df:
            if col == ref:
                continue
            x   = df[ref]
            y   = df[col]
            plt.scatter(x, y-x, marker=markers[i], edgecolor=colors[i],
                            label=False, c='none')
            i += 1
        plt.plot(l, l-l, color='black', linewidth=0.75, label=None)
        plt.tight_layout()
        plt.savefig(dir_path + 'Linear--Diff--All.png')
        if 'Bond' in name:
            plt.close()
        else:
            plt.yscale('symlog')
            plt.xscale('symlog')
            plt.savefig(dir_path + 'LogLog--Diff--All.png')
            plt.close()
#--------------------Combination Diff Plot----------------------------------

    return



from .config import *

def hes2raman(hesfile, datfile):
    """
    This function writes a raman calculation input file using a previously
    run hessian input file and the dat file generated by the calculation.

    Parameters
    ----------
    hesfile: string
        This should be a string that points to the input file of an
        already run hessian file. (FULL DIRECTORY STRING REQUIRED)
    datfile: string
        This should be a string that points to the DAT file of an
        already run hessian file. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    This function returns nothing if it terminates successfully, otherwise
    it returns ValueError.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> datfile = '../restart/IBv6_NH3_CCSD-T_CC6_hes.dat'
    >>> hesfile = './IBv6_NH3_CCSD-T_CC6_hes.inp'
    >>>
    >>> ag.hes2raman(hesfile, datfile)
    >>>
    """
    #Define Runtyps
    rhes   = '=HESSIAN'
    rraman = '=RAMAN'

    #Define grap list indexes
    s = ' $GRAD  \n'
    e = '----- HESSIAN MATRIX AFTER PROJECTION -----\n'

    #Define file identifiers
    hes   = '_hes'
    raman = '_raman'

    #Define raman file
    ramanfile = hesfile.replace(hes, raman)

    #Open, Read then close input file
    f   = open(hesfile, 'r')
    inp = f.readlines()
    f.close()

    #Open, Read then close DAT file
    f    = open(datfile, 'r')
    grab = f.readlines()
    f.close()

    #checks if they are in file
    if check_if_exists(datfile, ctr_f(s,grab), ctr_f(e,grab)):
        raise ValueError()

    #Define start and end indexes
    start = grab.index(s)
    end   = grab.index(e)

    #Replace HESSIAN with RAMAN
    i      = ctr_f(rhes, inp)
    inp[i] = inp[i].replace(rhes, rraman)

    #Open, Write then close raman file
    f = open(ramanfile, 'w')
    f.writelines(inp)
    f.writelines(grab[start:end])
    f.close()

    return



from .config import *
from .data_finder import *

class DATA:
    def __init__(self):
        return

def get_data(filename):
    """
    This function collects data from GAMESS(us) log files.

    Parameters
    ----------
    filename: string
        This should be a string that points to the log file of any
        GAMESS(us) calculation. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    data: object
        This is an object with all the data collected from the log file.
        Below is a list of the attributes associated with `data` based on
        each log file type.

        all   files: `cpu`, `time`
        opt   files: `bond_lengths`, `bond_angles`
        hes   files: `vib_freq`, `ir_inten`
        raman files: `raman`
        vscf  files: `vscf_freq`, vscf_ir

    Notes
    -------
    This function is primarily intended for interal use by AutoGAMESS.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> filename = './AGv0-0-6_NH3_CCSD-T_CC6_opt.log'
    >>>
    >>> ag.get_data(filename)
    >>>
    """
    data    = DATA()

    if '_opt' in filename:
        setattr(data, 'bond_lengths', optimization(filename)[0])
        setattr(data,  'bond_angles', optimization(filename)[1])

    if '_hes' in filename:
        setattr(data, 'vib_freq', hessian(filename)[0])
        setattr(data, 'ir_inten', hessian(filename)[1])

    if '_raman' in filename:
        setattr(data, 'raman', raman(filename))

    if '_vscf' in filename:
        setattr(data, 'vscf_freq', vscf(filename)[0])
        setattr(data,   'vscf_ir', vscf(filename)[1])

    if '_comp' in filename:
        setattr(data,   'hf_0k', composite(filename)[0])
        setattr(data, 'hf_298k', composite(filename)[1])

    setattr(data,  'cpu', comp(filename)[0])
    setattr(data, 'time', comp(filename)[1])

    return data



"""
This is the autogamess module, a python module designed to automate
optimization, hessian, raman, and VSCF input generation and output
data collection.

Contains the following functions:
                new_project
                opt2hes
                sort_logs
                sort_inps
                hes2raman
                input_builder
                fill_spreadsheets
                get_data
                make_plot
                generate_scaling_factors
                convert_ir_units
                make_histogram
                make_scatter


Author: Brian C. Ferrari
"""
    #Configuration (holds Globals)
from .config        import *
from .classes       import *

    #Functions
from .new_project              import new_project
from .opt2hes                  import opt2hes
from .sort_logs                import sort_logs
from .sort_inps                import sort_inps
from .hes2raman                import hes2raman
from .input_builder            import input_builder
from .fill_spreadsheets        import fill_spreadsheets
from .get_data                 import get_data
from .make_plot                import make_plot
from .generate_scaling_factors import generate_scaling_factors
from .convert_ir_units         import convert_ir_units

    #Multi-function Sub Packages
from .data_finder       import *
from .plots             import *



from .config import *
from openpyxl import load_workbook

def convert_ir_units(old, new, sheetsdir):
    """
    """
    #Define Sheet names
    opt = 'Optimization'
    hes = 'Hessian'
    ram = 'Raman'
    vsc = 'VSCF'

    xlsx = '.xlsx'
    CF   = conversion_factor(old, new)

    for file in os.listdir(sheetsdir):

        df = pd.read_excel(sheetsdir + file, index_col=0,
                           sheet_name=None, header=6)

        temp = df[hes].filter(regex='Infrared Intensity') * CF
        df[hes].update(temp)

        #Write Spreadsheets
        book = load_workbook(sheetsdir + file)
        with pd.ExcelWriter(sheetsdir + file, engine='openpyxl') as writer:
            writer.book = book
            writer.sheets = dict((ws.title, ws) for ws in book.worksheets)
            if opt in df:
                df[opt].to_excel(writer, sheet_name=opt, startrow=6)
            if hes in df:
                df[hes].to_excel(writer, sheet_name=hes, startrow=6)
            if ram in df:
                df[ram].to_excel(writer, sheet_name=ram, startrow=6)
            if vsc in df:
                df[vsc].to_excel(writer, sheet_name=vsc, startrow=6)


    return



from ..config       import *
from ..dictionaries import *
import elements.elements as el
import basis_set_exchange as bse
from copy import deepcopy

#----------------------PROJECT CLASS--------------------------------------
class PROJECT:
    'AutoGAMESS Project Class'

    def __init__(self):
        self.title      = ''
        self.species    = []
        self.theories   = []
        self.comp_meths = []
        self.basis_sets = []
        self.ext_basis  = []
        self.run_types  = []
        self.map        = {}

    def make_project(self, maindir, safety_check=False):
        """
        This function creates a new directory tree for a GAMESS project, also makes
        a couple of text files for use with other functions.

        Parameters
        ----------
        maindir: string
            A directory string (including the final `/`) that points to the
            directory that the project tree will be spawned in.
        safety_check: boolean True/False [Optional]
            if True then input files will be internal reviewed by AutoGAMESS.

        Returns
        ----------
        This function returns nothing

        Notes
        ----------
        The format of the spawned directory tree is as follows:

                                        maindir
                                           |
                                         title
                            -------------------------------------
                            |        |        |        |        |
                          Codes    Inps     Logs  Batch_Files  Spreadsheets
                            |        |        |                     |
                      ---------    Block    -----------      1 file per specie
                      |       |             |    |    |
                Text_Files  Scripts      Fail   Pass  Sorted
                                          |      |        |
                                       -------  Block   1 directory per specie
                                       |     |
                                 Unsolved   Solved


        Sections in directory tree labled 'Block' are directory trees with the
        following format:
                          1 directory per run type
                                      |
                          1 directory per specie
        """
        self.make_dir_tree(maindir)
        projdir = maindir + self.title + '/'
        self.build_inps(projdir, safety_check=safety_check)

    def make_dir_tree(self, maindir):
        #Defining directory names
        unsolved = maindir + self.title + '/Logs/Fail/Unsolved/'
        solved   = maindir + self.title + '/Logs/Fail/Solved/'
        xldir    = maindir + self.title + '/Spreadsheets/'
        inputs   = maindir + self.title + '/Inps/'
        goodlogs = maindir + self.title + '/Logs/Pass/'
        sorrted  = maindir + self.title + '/Logs/Sorted/'

        #Define random commands
        fin      = '\nBREAK\n'
        engine   = 'xlsxwriter'
        xlsx     = '.xlsx'

        #Make directories
        try:
            os.makedirs(unsolved)
            os.makedirs(solved)
            os.makedirs(xldir)

            #Make Block directory trees
            for runtyp in self.run_types:
                runtyp += '/'
                for specie in self.species:
                    specie += '/'
                    os.makedirs(inputs+runtyp+specie)
                    os.makedirs(goodlogs+runtyp+specie)
        except:
            print(error_head)
            print("One or more directories in Project tree already exist")
            print(error_tail)

        #Make data for Spreadsheets
        theo = []
        for theory in self.theories:
            temp    = [theory]*len(self.basis_sets + self.ext_basis)
            theo   += temp + ['\n', '\n','\n']

        bs = (self.basis_sets + self.ext_basis
              + ['\n', '\n','\n']) *len(self.theories)

        #Make dataframe with theory and basis sets names only
        data = {   'Theory': theo,
                'Basis Set': bs}
        df2  = pd.DataFrame(data=data)

        #Make data for Composite method Sheets
        data = {'Method': self.comp_meths}
        df3  = pd.DataFrame(data=data)

        #More directory making and Excell workbook and sheet making
        for specie in self.species:
            specie += '/'
            os.makedirs(sorrted+specie)

            #Define header for Spreadsheets
            header = [version, author, '',
                      'Project Name : ' +  self.title,
                      'Molecule Name: ' + specie.replace('/', '')]

            #Define Excell filename
            xlfilename = xldir + specie.replace('/', xlsx)

            #Initialize writer
            writer = pd.ExcelWriter(xlfilename, engine=engine)

            #Make Sheets and put headers on them all
            for runtyp in self.run_types:
                runtyp += '/'

                #Define sheet name and make it
                if runtyp == 'Composite/':
                    sheet     = runtyp.replace('/', '')
                    df3.to_excel(writer, startrow=6, startcol=0, sheet_name=sheet)
                    worksheet = writer.sheets[sheet]
                else:
                    sheet     = runtyp.replace('/', '')
                    df2.to_excel(writer, startrow=6, startcol=0, sheet_name=sheet)
                    worksheet = writer.sheets[sheet]

                #Write Header
                for line in header:
                    i = header.index(line)
                    worksheet.write(i, 0, line)

                #Write Units in header
                u  = 'Units:'
                bl = 'Bond Length (Å)'
                ba = 'Bond Angle (degrees)'
                vf = 'Vibrational Frequency (cm⁻¹)'
                ii = 'Infrared Intensity (km mol⁻¹)'
                ra = 'Raman Activity (angstrom⁴ amu⁻¹)'
                fv = 'VSCF Frequency (cm⁻¹)'
                iv = 'VSCF IR (km mol⁻¹)'
                hf = 'Heat of Formation (kcal mol⁻¹)'
                sb = '          '
                if runtyp == 'Optimization/':
                    worksheet.write(2, 0, u + sb + bl + sb + ba)
                if runtyp == 'Hessian/':
                    worksheet.write(2, 0, u + sb + vf + sb + ii)
                if runtyp == 'Raman/':
                    worksheet.write(2, 0, u + sb + ra)
                if runtyp == 'VSCF/':
                    worksheet.write(2, 0, u + sb + fv + sb + iv)
                if runtyp == 'Composite/':
                    worksheet.write(2, 0, u + sb + hf)

            #Save Excell file
            writer.save()

    def make_file_name(self, specie, inp):
        theory = inp.theory
        basis  = inp.basis_set

        if len(inp.Contrl.runtyp) > 5:
            if inp.Contrl.runtyp == 'COMPOSITE':
                run_name = 'comp'
            run_name  = inp.Contrl.runtyp[0:3].lower()
        else:
            run_name  = inp.Contrl.runtyp.lower()

        file_name = agv + '_' + specie + '_' + theory + '_'  + \
                    basis + '_' + run_name + '.inp'

        return file_name

    def build_inps(self, savedir, safety_check=False):

        #Molecule Iterator
        for specie in self.species:

            #Theory Level Iterator
            for theory in self.theories:

                #Internal Basis Set Iterator
                for basis in self.basis_sets:
                    inp            = deepcopy(self.map[specie])   #Get Template
                    inp.Data.title = self.title                   #Add Title
                    inp.prep_template()                           #Prep Template
                    inp.theory = theory                           #Def Theory Global
                    theo = theory_dict[theory].split('=')         #Prep it
                    setattr(inp.Contrl, theo[0].lower(), theo[1]) #Add it

                    inp.basis_set    = basis #Define Basis Set Global
                    inp.Basis.gbasis = basis #Add Basis Set to Inp

                    #Delete external basis set
                    if inp.Data.basis:
                        inp.Data.basis = {}

                    #Run type and file naming
                    inp.run_type = inp.Contrl.runtyp.lower()
                    file_name    = self.make_file_name(specie, inp)

                    if safety_check:
                        inp.check()
                    inp.write_inp(savedir + file_name)


                #External Basis Set Iterator
                for basis in self.ext_basis:
                    inp            = deepcopy(self.map[specie])   #Get Template
                    inp.Data.title = self.title                   #Add Title
                    inp.prep_template()                           #Prep Template
                    inp.theory = theory                           #Def Theory Global
                    theo = theory_dict[theory].split('=')         #Prep it
                    setattr(inp.Contrl, theo[0].lower(), theo[1]) #Add it

                    #Delete GAMESS(US) internal Basis Set group
                    if hasattr(inp, 'Basis'):
                        delattr(inp, 'Basis')

                    inp.basis_set = basis
                    file_name     = self.make_file_name(specie, inp)
                    inp.Data.add_basis(basis)

                    if safety_check:
                        inp.check()
                    inp.write_inp(savedir + file_name)

                #Remove External Basis set from inp
                inp.Data.basis = {}



from ..config      import *
from ..dictionaries import *
import elements.elements as el
import basis_set_exchange as bse

#Class for input parameters
class INPUT:
    'GAMESS(US) input parameters'

    theory    = ''
    basis_set = ''
    run_type  = ''

    class Param_Group:
        'Subclass for GAMESS(US) input parameter groups'

        def __init__(self, name):
            self.name = name

        def make_string(self):
            s = ' $' + self.name.upper() + ' '
            e = '$END\n'
            n = len(s)
            for i in self.__dict__:
                if i == 'name':
                    continue
                add_this = i.upper() + '=' + self.__dict__[i] + ' '
                if n + len(add_this) > 50:
                    s += '\n '
                    n  = 0
                s += add_this
                n += len(add_this)

            s += e
            return s

    class Data_Group:
        'Subclass for GAMESS(US) Data Input group'

        def __init__(self):
            self.title  = ''
            self.sym    = None
            self.coords = []
            self.basis  = {}

        def read_inp(self, inp):
            i = inp.split('$DATA')[1].split('\n')
            s = ctr_f(  '.0', i)
            e = ctr_f('$END', i)
            self.sym    = i[2]
            self.title  = i[1]

            n = False
            for line in i[3:]:
                j = len(line.split())
                if ((j == 5) and n) or (('$END' in line) and n):
                    m    = i.index(line) - 1
                    if m > n:
                        self.basis[atom] = '\n'.join(i[n:m])
                        n    = False
                if j == 5:
                    self.coords.append(line)
                    atom = line.split()[0]
                    n    = i.index(line) + 1


        def get_elements(self):
            elements = []
            for line in self.coords:
                x = line.split()[0]
                if x not in elements:
                    elements.append(x)

            return elements

        def add_basis(self, basis_name):
            if basis_name in basis_dict:
                basis_name = basis_dict[basis_name]

            elements  = self.get_elements()
            atoms     = [getattr(el,x) for x in elements]
            basis_set = bse.get_basis(basis_name, elements=elements,
                                      fmt='gamess_us', header=False)

            atoms.sort(key=lambda x: x.AtomicNumber)
            for i in range(len(atoms)):
                name1 = atoms[i].Name.upper()

                try:
                    name2 = atoms[i+1].Name.upper()
                except:
                    name2 = '$END'

                a                           = basis_set.find(name1) + len(name1)
                b                           = basis_set.find(name2)
                atom_basis                  = basis_set[a:b]
                self.basis[atoms[i].Symbol] = atom_basis[1:].strip('\n')

            return

        def make_string(self):
            s = ' $DATA\n'

            if self.title:
                if self.title[0] != ' ':
                    s += ' ' + self.title + '\n'
                else:
                    s += self.title + '\n'

            e = '$END'

            if '\n' in self.sym:
                s += self.sym
            else:
                if 'C1' in self.sym:
                    s += self.sym + '\n'
                else:
                    s += self.sym + '\n\n'


            if self.basis:
                for line in self.coords:
                    atom = line.split()[0]
                    if '\n' not in line:
                        line += '\n'
                    s   += line + self.basis[atom] + '\n\n'
                s += e
                return s

            if '\n'not in self.coords:
                s += '\n'.join(self.coords)
                s += '\n'
            else:
                s += ''.join(self.coords)
            s += e
            return s


    #--------------------------INPUT funtions--------------------------------
    def __init__(self, template):

        if template in templates:
            inp = templates[template]
        else:
            inp  = read_file(template)
            inp  = ''.join(inp)
            data = inp
            inp  = inp.replace('\n', '')

        inp = inp.split('$END')
        for i in inp:
            if '$' not in i:
                continue

            if '$DATA' in i:
                continue

            i                             = i.split('$')[1].split()
            group_name                    = i[0].capitalize()
            params                        = i[1:]
            exec('self.' + group_name + ' = self.Param_Group(group_name)')
            for j in params:
                j   = j.split('=')
                key = j[0].lower()
                val = j[1]
                exec('setattr(self.' + group_name + ', key, val)')

        self.Data = self.Data_Group()
        self.Data.read_inp(data)

    def write_inp(self, file_name):
        f = open(file_name, 'w')
        f.write('!'+ version +'\n')
        f.write('!  by Brian C. Ferrari \n')
        f.write('!\n')

        for i in self.__dict__:
            if 'Data' in i:
                continue

            try:
                f.write(getattr(self, i).make_string())
            except:
                continue

        try:
            f.write(getattr(self, 'Data').make_string())
        except:
            pass

        f.close()

    def prep_template(self):
        #Remove Theory Level command form input template
        stuff2delete = ['dfttyp', 'cctyp', 'mplevl']
        for i in stuff2delete:
            try:
                delattr(self.Contrl, i)
            except:
                pass

        #Remove Basis Set group from input template & generate blank one
        try:
            delattr(self, 'Basis')
            self.Basis = self.Param_Group('Basis')
        except:
            self.Basis = self.Param_Group('Basis')

    def check(self):
        #Remove spherical coordinates if Pople basis set in use
        if '6-31' in self.basis_set:
            try:
                delattr(self.Contrl, 'ispher')
            except:
                pass

        #Ensure CC types have FULLNUM and numgrd
        if hasattr(self.Contrl, 'cctyp'):
            self.Contrl.numgrd = '.T.'
            if not hasattr(self, 'Force'):
                self.Force = self.Param_Group('Force')
            self.Force.method = 'FULLNUM'
            self.Force.nvib   = '2'
            self.Force.projct = '.T.'

        #Add force card for hessians
        if self.Contrl.runtyp == 'HESSIAN':
            if not hasattr(self, 'Force'):
                self.Force = self.Param_Group('Force')
            self.Force.method = 'SEMINUM'
            self.Force.nvib   = '2'
            self.Force.projct = '.T.'

            if hasattr(self.Contrl, 'numgrd'):
                delattr(self.Contrl, 'numgrd')

            if hasattr(self.Contrl, 'cctyp'):
                self.Force.method = 'FULLNUM'

        #Inlcude Jans=2 grid for all DFT functionals
        if hasattr(self.Contrl, 'dfttyp'):
            if  not hasattr(self, 'Dft'):
                self.Dft      = self.Param_Group('Dft')
                self.Dft.jans = '2'
            else:
                self.Dft.jans = '2'

        #Ensure that SCS and IMS commands are included for SCS-MP2
        if 'SCS-MP2' in self.theory:
            if not hasattr(self, 'Mp2'):
                self.Mp2       = self.Param_Group('Mp2')
                self.Mp2.scspt = 'SCS'
                self.Mp2.code  = 'IMS'
            else:
                self.Mp2.scspt = 'SCS'
                self.Mp2.code  = 'IMS'

        #Raise max iteration of CCSD and left eigenstate for CCSD(2)T
        if 'CCSD2-T' in self.theory:
            if not hasattr(self, 'Ccinp'):
                self.Ccinp        = self.Param_Group('Ccinp')
                self.Ccinp.maxcc  = '100'
                self.Ccinp.maxccl = '100'
            else:
                self.Ccinp.maxcc  = '100'
                self.Ccinp.maxccl = '100'

        #Remove DFT group from non-DFT calculations
        if (not hasattr(self.Contrl, 'dfttyp')) and (hasattr(self, 'Dft')):
            delattr(self, 'Dft')

        #Remove MP2 group from non-MP calculations
        if (not hasattr(self.Contrl, 'mplevl')) and (hasattr(self, 'Mp2')):
            delattr(self, 'Mp2')

        #Remove CCinp group from non-CC calculations
        if (not hasattr(self.Contrl, 'cctyp')) and (hasattr(self, 'Ccinp')):
            delattr(self, 'Ccinp')



from ..config import *
import matplotlib.pyplot as plt
from ..dictionaries  import theory_dict


def make_kde(data, savedir):
    """
    """
    #Define Sheet names
    opt = 'Optimization'
    hes = 'Hessian'
    ram = 'Raman'
    vsc = 'VSCF'

    #Define Data Column names/variables
    rt = 'Run Time'
    bs = 'Basis Set'
    cp = 'CPU Percentage'
    te = 'Theory'
    me = 'Method'
    fe = 'Freq'
    bl = 'Bond Length'
    ir = 'Infrared'
    ra = 'Raman'

    #Other predefined strings
    png      = '.png'
    xlsx     = '.xlsx'
    engine   = 'xlsxwriter'
    X        = 'X'

    #Predefine Markers and Colors
    colors   = ['r','k','blue','darkorange', 'lime','olive', 'deepskyblue',
                'purple', 'gold']
    styles   = ['-', '-', '-', '-', '-', '-', '-', '--', '-']


    if xlsx in data:
        data = pd.read_excel(data,  index_col=0, sheet_name=None, header=6)

    for name, df in data.items():
        ref = df.columns[0]
        dir_path = savedir + name + '/'

        try:
            os.makedirs(dir_path)
        except:
            pass

        for col in df:
            if col == ref:
                continue

            x = df[ref]
            y = df[col]
            df[col] = y-x


        df.drop(ref, axis=1, inplace=True)

        df.dropna(thresh=5, axis=1).plot.kde(color=colors, style=styles)
        plt.tight_layout()
        plt.savefig(dir_path + 'KDE.png')
        plt.close()

    return



from .config import *
from openpyxl import load_workbook

def double_check_sheet(projdir):
    """
    """
    #string variables
    sheetsdir      = projdir + 'Spreadsheets/'
    xlsx           = '.xlsx'
    hes            = 'Hessian'

    for specie in species:
        if os.path.isfile(sheetsdir + specie + xlsx):
            df  = pd.read_excel(sheetsdir + specie + xlsx, index_col=0,
                               sheet_name=hes, header=6)
                               
    return



from .config import *
from .classes.INPUT import INPUT

def opt2hes(optfile, logfile):
    """
    This function writes a hessian calculation input file using a previously
    run optimization input file and the log file generated by the calculation.

    Parameters
    ----------
    optfile: string
        This should be a string that points to the input file of an
        already run optimization file. (FULL DIRECTORY STRING REQUIRED)
    logfile: string
        This should be a string that points to the log file of an
        already run optimization file. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    This function returns nothing if it terminates successfully, otherwise
    it returns ValueError.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> logfile = './Optimization_Log_Folder/IBv6_NH3_CCSD-T_CC6_opt.log'
    >>> optfile = './IBv6_NH3_CCSD-T_CC6_opt.inp'
    >>>
    >>> ag.opt2hes(optfile, logfile)
    >>>
    """
    #Define file identifiers
    opt = '_opt'
    hes = '_hes'

    #Make INPUT object
    inp = INPUT(optfile)

    #Open, read in, and close log file
    log = read_file(logfile)

    #Grabs optimized geometries tail index
    tfind = 'COORDINATES OF ALL ATOMS ARE'
    dtail = len(log) - ctr_f(tfind, log[::-1]) - 1
    if (hasattr(inp.Contrl, 'nosym')) and (inp.Contrl.nosym == '1'):
        tfind = 'INTERNUCLEAR DISTANCES (ANGS.)'
        dtail = len(log) - ctr_f(tfind, log[::-1]) - 2

    #Grabs optimized geometries header index
    hfind = '***** EQUILIBRIUM GEOMETRY LOCATED *****'
    dhead = ctr_f(hfind, log) + 4

    #Checks to make sure head and tail exist
    if check_if_exists(logfile, ctr_f(hfind, log)):
        raise ValueError()

    #Assemble list of optimized geometry coordinates and get size
    coords = log[dhead : dtail]

    #Change run type
    inp.Contrl.runtyp = 'HESSIAN'

    #Change atomic coordinates
    inp.Data.coords = coords

    #Save new file
    inp.check()
    inp.write_inp(optfile.replace(opt, hes))

    return



from .config import *

def sort_inps(projdir, inpsdir):
    """
    This function sorts all the loose log files in the 'Logs' directory.

    Parameters
    ----------
    projdir: string
        A directory string (including the final `/`) that points to the
        project head directory.
    inpsdir: string
        A directory string (including the final `/`) that points to the
        directory containing the input files.

    Returns
    ----------
    This function returns nothing

    Notes
    ----------
    For this function to work properly the project directory tree must be
    in the exact format that the 'new_project' fucntion spawned it in.

    Examples
    ----------
    >>>import sort_logs
    >>>projdir = './Example/'
    >>>inpdir  = './Inps/'
    >>>sort_inps(projdir, inpsdir)
    >>>
    """
    #Defining Shorthand to directory name dictionary
    typ_dict = {'opt'  : 'Optimization/',
                'hes'  : 'Hessian/',
                'raman': 'Raman/',
                'vscf' : 'VSCF/',
                'comp' : 'Composite/'}

    #Defining directory names
    dir      = projdir + 'Inps/'

    #Checks if Inps dir is real directory
    if not os.path.isdir(dir):
        dir = projdir

    #Defining extension names
    ipext    = '.inp'

    for filename in os.listdir(inpsdir):

        #Skips non-inp files
        if ipext not in filename:
            continue

        #Get file runtype, then puts string in directory format
        typ0 = filename.split('_')[-1].split('.')[0]
        if typ0[-1].isnumeric():
            typ0 = typ0[0:-1]
        typ  = typ_dict[typ0]

        #Gets molecule name, then puts string in directory format
        specie  = filename.split('_')[1]
        specie += '/'

        #Define the directory to put this particular file in
        move2 = dir + typ + specie

        #Checks if move2 directory exists, if not then makes it
        if not os.path.isdir(move2):
            os.makedirs(move2)

        #Define `before/after` of rename command to move file
        before = inpsdir + filename
        after  = move2   + filename

        #Moves log to proper directory
        os.rename(before, after)


    return



from .config import *
import pkg_resources
from .dictionaries import *
import elements.elements as el

def input_builder(inputfile, save_dir, initial_coords_dict=None,
                  proj_title=' Your Title Goes Here\n'):
    """
    This function builds optimization input files.

    Parameters
    ----------
    inputfile: string
        This should be a full directory string that points to the input
        csv file.
    save_dir: string
        this should be a full directory string that points to the directory
        you wish to save the inputs in.
    initial_coords_dict: dictionary [Optional]
        This should be a dictionary with the key being the specie and the
        value being a list of the symmetry group and symmetry unique atom
        coordinates. Examples can be seen in the AutoGAMESS GitHub repository
        as well as by prinint out the default dictionary, see Notes section.
    proj_title: string [Optional]
        This should be a string ending with `\n`

    Notes 1
    ----------
    If the molecules you wish to build are not already defined in the
    general autogamess coordinate dictionary, then initial_coords_dict
    must be passed.

    To see the autogamess coordianate dictionary simply print out
    >>> ag.dictionaries.molecule_dictionary

    Returns
    ----------
    This function returns nothing

    Notes 2
    ----------
    This function uses the EMSL Basis Set Exchange module to import
    external basis sets[1]. This function also uses the Periodic_Elements
    package by VaasuDevanS [2].

    [1] https://github.com/MolSSI-BSE/basis_set_exchange
    [2] https://github.com/VaasuDevanS/Periodic_Elements


    Examples
    ----------
    >>> import autogamess as ag
    >>>
    >>> csvfile = './input.csv'
    >>> savedir = './'
    >>> title   = 'Project\n'
    >>>
    >>> ag.input_builder(csvfile, savedir, proj_title=title)
    >>>
    """
    #Variable Declrations
    _       = '_'
    version = (' AutoGAMESS Version ' +
                str(pkg_resources.require("autogamess")[0].version) )
    ibv     = 'AGv' + version.split(' ')[-1].replace('.', '-')
    opt     = '_opt.inp'
    cmp     = '_comp.inp'
    numgrd  = 'NUMGRD=.T.'

    #Checks if initial_coords_dict is given
    if initial_coords_dict is None:
        initial_coords_dict = molecule_dictionary

    #Read in input file
    df = pd.read_csv(inputfile)

    #Get list of species/theories/basis sets
    species     = list(df['Species'].dropna())
    theories    = list(df['Theory'].dropna())
    cmethods    = list(df['Composite Methods'].dropna())
    basis_sets  = list(df['Basis Sets'].dropna())
    ebasis_sets = list(df['External Basis Sets'].dropna())

    #Make list of file names
    files = []
    for specie in species:
        for theorie in theories:
            for ebasis in ebasis_sets:
                files.append( ibv+ _ + specie.strip('\n') + _ +
                              theorie.strip('\n') + _ +
                              ebasis.strip('\n') + opt )
            for basis in basis_sets:
                files.append( ibv+ _ + specie.strip('\n') + _ +
                              theorie.strip('\n') + _ +
                              basis.strip('\n') + opt )
        for cmethod in cmethods:
            files.append( ibv+ _ + specie.strip('\n') + _ +
                          cmethod.strip('\n') + cmp )

    #Make input files for GAMESS(us)
    for filename in files:
        #Open file, write preamble
        f = open(save_dir + filename, 'w')
        f.write('!'+ version +'\n')
        f.write('!  by Brian C. Ferrari \n')
        f.write('!\n')

        #Composit method input building
        if cmp in filename:
            theo      = comp_dict[filename.split(_)[2]]
            i         = ctr_f('theory', comp_basic_params)
            params    = list(comp_basic_params)
            params[i] = comp_basic_params[i].replace('theory', theo)

            if 'G4MP2' in theo:
                params.insert(ctr_f('$COMP', params)+1,
                              ' $FORCE METHOD=SEMINUM $END\n')
            if 'CCCA-S4' in theo:
                i = ctr_f('DAMP=.T.', params)
                del params[i]
                i = ctr_f('$SCF', params)
                params[i].replace('DIRSCF=.TRUE. FDIFF=.FALSE. CONV=1d-7\n',
                                  'DAMP=.T. CONV=1d-7 $END\n')

            #Write parameters, and project title
            f.writelines(params)
            f.write(proj_title)

            #Get initial coordinates from initial_coords_dict
            coords = initial_coords_dict[filename.split(_)[1]]
            f.writelines(coords)
            f.write('$END')
            f.close()
            continue

        #Get index of 'theory' to replace with proper command
        i               = ctr_f('theory', basic_params)

        #Get proper theory input style from dictionary
        theo            = theory_dict[filename.split(_)[2]]

        #Get list version of tuple basic parameters
        #(its a tuple to protect it from changes)
        params          = list(basic_params)

        #Replace 'theory' with proper input parameter
        params[i]       = basic_params[i].replace('theory', theo)

        #Add numerical gradients in cctyps if not already there
        if 'CCTYP=' in theo:
            if ctr_f(numgrd, params) == -1:
                params[i] += ' NUMGRD=.T.'

        #Add extra input lines needed for SCS-MP2 and CCSD2-T
        if 'CCSD2-T' in filename.split(_)[2]:
            params.insert(ctr_f('$DATA', params)-1,
                          ' $CCINP MAXCC=100 MAXCCL=100 $END\n')
        if 'SCS' in filename.split(_)[2]:
            params.insert(ctr_f('$DATA', params)-1,
                          ' $MP2 SCSPT=SCS $END\n')

        #Fill in basis parameter for GAMESS(us) internal basis sets
        if filename.split(_)[3] in basis_sets:
            i               = ctr_f('=basis', params)
            bset            = filename.split(_)[3]
            if '6-31' in bset:
                pople = (' $BASIS GBASIS=N' + bset.split('-')[1] +
                         ' NGAUSS=6 NDFUNC=' + bset.split('-')[3].strip('d')+
                         ' NPFUNC=' + bset.split('-')[4].strip('p')+
                         ' $END\n')
                if 'pG' in bset:
                    pople = (' $BASIS GBASIS=N' + bset.split('-')[1] +
                             ' NGAUSS=6 NDFUNC=' + bset.split('-')[3].strip('d')+
                             ' NPFUNC=' + bset.split('-')[4].strip('p')+ '\n'+
                             ' DIFFSP=.TRUE. $END\n')
                if 'ppG' in bset:
                    pople = (' $BASIS GBASIS=N' + bset.split('-')[1] +
                             ' NGAUSS=6 NDFUNC=' + bset.split('-')[3].strip('d')+
                             ' NPFUNC=' + bset.split('-')[4].strip('p')+ '\n'+
                             ' DIFFS=.TRUE. DIFFSP=.TRUE. $END\n')
                params[i] = pople
            else:
                params[i]       = params[i].replace('basis', bset)
        #If not internal delete basis line from input file
        else:
            i               = ctr_f('=basis', params)
            del params[i]

        #Write parameters, and project title
        f.writelines(params)
        f.write(proj_title)

        #Get initial coordinates from initial_coords_dict
        coords = initial_coords_dict[filename.split(_)[1]]

        #Using EMSL put in external basis sets
        if filename.split(_)[3] in ebasis_sets:
            basis_name = filename.split(_)[3]
            molecule   = filename.split(_)[1]
            elements   = get_elements(molecule)
            atoms      = [getattr(el,x) for x in elements]
            if basis_name in basis_dict:
                basis_name = basis_dict[basis_name]
            basis = bse.get_basis(basis_name, elements=elements,
                                  fmt='gamess_us', header=False).split('\n')[1:]
            basis[0] = coords[0]

            for atom in atoms:
                name = atom.Name.upper()
                symb = atom.Symbol + str(atom.AtomicNumber)
                i = ctr_f(name, basis) + 1
                x = []
                while (basis[i] != '') and (basis[i] != '$END'):
                    x.append(basis[i]+'\n')
                    i+=1

                tmp = coords.copy()
                j   = 1
                for i in range(len(coords)):
                    if symb == coords[i].split('.')[0].replace(' ',''):
                        tmp[i+j:i+j] = x
                        j+=len(x)
                        tmp[i+j:i+j] = '\n'
                        j+=1
                coords = tmp

        f.writelines(coords)
        f.write('$END')
        f.close()

    return



from .config import *
from openpyxl import load_workbook

def generate_scaling_factors(projdir, expt_dict, species):
    """
    This function generates scaling factors and scaled frequencies.

    Parameters
    ----------
    projdir: string
        This should be a full directory string pointing to the project
        directory initlly created by new_project.
    expt_dict: dictionary
        This should be a python dictionary with the experimental frequency
        values for all species that the user wants to generate scaling factors
        for in it. Format is explained in Notes section.
    species: list
        This should be a list of all species the user would like scaling factors
        generated for. Any molecule in the list must have experimental data in
        the `expt_dict` associated with it.

    Notes
    -------
    `expt_dict` format should be as follows:

            {`specie`: [`nu_1`, `nu_2`, ... , `nu_N`]}

     where `specie` must be written the same way as the Excel spreadsheet file
     for that molecule is written. Each frequency, `nu`, should be given in
     the same order as they appear (left to right) in the spreadsheet.

     `species` list format can be in any order but must adhere to the rule
     that any element in `species` is a key for `expt_dict`

     Once execution of this function is completed the `Hessian` worksheet
     will be updated to have a coulmn giving `Scaling Factor/RMS`, as well
     as the scaled frequencies will appear in parathesis next to the predicted
     frequencies.

    Returns
    -------
    This function returns nothing.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> projdir   = './Your Project Title/'
    >>> expt_dict = {'H2O': [1595, 3657, 3756]}
    >>> species   = ['H2O']
    >>>
    >>> ag.generate_scaling_factors(projdir, expt_dict, species)
    >>>
    """
    #string variables
    sheetsdir      = projdir + 'Spreadsheets/'
    xlsx           = '.xlsx'
    hes            = 'Hessian'

    for specie in species:
        if os.path.isfile(sheetsdir + specie + xlsx):
            df  = pd.read_excel(sheetsdir + specie + xlsx, index_col=0,
                               sheet_name=hes, header=6)
            df2 = pd.read_excel(sheetsdir + specie + xlsx, index_col=0,
                               sheet_name=hes, header=6)

        #get only frequency columns
        x = [col for col in df.columns if 'Vibrational' not in col]
        df.drop(x, axis=1, inplace=True)
        df.dropna(inplace=True)

        #get expt data from dictionary
        expt = expt_dict[specie]

        df2['Scaling Factor/RMS'] = np.nan
        #iterate through DataFrame
        for i,j in df.iterrows():
            theo = list(j.values)

            #get scaling factors
            c, rms = scaling_factor_scott(theo, expt)

            #apply to columns in sliced DataFrame
            df.loc[i] = j.apply(lambda x: str(x) + '(' + str(round(float(x)*c, 2)) + ')')

            #Added column giving Scaling Factor and RMS
            df2.loc[i,'Scaling Factor/RMS'] = str(c) + '/' + str(rms)


        #update main DataFrame
        df2.update(df)

    #write Excel spreadsheet with updated DataFrame
    book = load_workbook(sheetsdir + specie + xlsx)
    with pd.ExcelWriter(sheetsdir + specie + xlsx, engine='openpyxl') as writer:
        writer.book = book
        writer.sheets = dict((ws.title, ws) for ws in book.worksheets)
        df2.to_excel(writer, sheet_name=hes, startrow=6)

    return



from .config import *
from .get_data import get_data
from .data_finder import hessian
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

def make_plot(file, savedir=None, cmap=['b', 'k', 'r', 'g', 'y', 'c'],
              method=None, sig=300, flag=[], reverse_x=True):
    """
    This function make vibrational frequency vs. IR/Raman intensity line plots.

    Parameters
    ----------
    file: string
        This should be a string that points to the log file of a hessian or
        Raman GAMESS(us) calculation. (FULL DIRECTORY STRING REQUIRED)
    savedir: string
        This should be a string that points to the directory in which you
        would like to save the png of the plot.(FULL DIRECTORY STRING REQUIRED)
    cmap: list [Optional]
        This should be a list of Matplotlib allowed color choices. Each symmetry
        will be plotted with a different color in the list.
    method: string [Optional]
        This should be string giving the method for line broadening, options are
        `Gaussian`, `Lorentzian`, None(defualt).
    sig: integer or float [Optional]
        This should be a numerical value to be used as the FWHM for the line
        broadening method chosen. Default: 300 wavenumbers
    flag: list [Optional]
        This should be a list of integers, in particular 1,2 and 3. This list
        tells the function what to plot and what to omit from the plot.
        Please see the Notes section for more details.
    reverse_x: boolean True/False [Optional]
         if True then x-axis will be in reverse (ie: 300---150----0).

    Notes
    -------
    The `flag` parameter is used as follows:

    [Default] `flag=[]`    ---> All lines are plotted
              `flag=[1]`   ---> Vertical lines are not plotted
              `flag=[2]`   ---> Spectral line not plotted
              `flag=[3]`   ---> Gaussian/Lorentzian lines not plotted
              `flag=[1,2]` --->V Vertical lines and Spectral line not plotted

    List combination follow the same format, all possible list combinations
    are allowed.

    Returns
    -------
    This function returns nothing.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> file    = './AGv0-0-6_NH3_CCSD-T_CC6_hes.log'
    >>> savedir = './'
    >>>
    >>> ag.make_plot(file, savedir)
    >>>
    >>> cmap = ['b', 'r', 'k', 'c']
    >>> ag.make_plot(file, savedir, cmap=cmap)
    >>>
    >>> method = 'Lorentzian'
    >>> sig    = 450
    >>>
    >>> ag.make_plot(file, savedir, cmap=cmap, method=method, sig=sig)
    >>>
    """
    #variables declarations
    png  = '.png'

    #get data from file
    data  = get_data(file)

    #Conversion factor
    cf = conversion_factor('Debye^2 angstrom^-2 amu^-1', 'km mol^-1')

    #If hessian file, make Ir vs Vib Freq plot
    if '_hes' in file:
        ir = data.ir_inten
        vf = data.vib_freq

        #init plot plane
        fig, ax = plt.subplots()

        #get x max
        x_max = 0
        for key in vf:
            x = np.float_(vf[key])
            if max(x) > x_max:
                x_max = max(x)

        #iterate through data making lines
        i    = 0
        x2   = np.arange(0, x_max*1.25, 0.01)
        sum  = np.zeros(len(x2))
        for key in vf:
            x = np.float_(vf[key])
            y = np.float_(ir[key])*cf

            #check flag, plot v lines
            if 1 not in flag:
                ax.vlines(x, 0, y, label=key, colors=cmap[i])
            if (1 in flag) and (3 not in flag):
                ax.vlines(0, 0, 0, label=key, linestyle='--', colors=cmap[i])

            #Make Gaussian line broadening
            if method == 'Gaussian':
                for a,b in zip(x,y):
                    gfit = gaussian(x2, a, sig, b)
                    sum += gfit

                    #Check flag, plot fits
                    if 3 not in flag:
                        ax.plot(x2, gfit, linestyle='--', color=cmap[i])

            #Make Lorentzian line broadening
            if method == 'Lorentzian':
                for a,b in zip(x,y):
                    lfit = lorentzian(x2, a, sig, b)
                    sum += lfit

                    #Check flag, plot fits
                    if 3 not in flag:
                        ax.plot(x2, lfit, linestyle='--', color=cmap[i])

            i += 1

        #check flag, plot fit sum
        if (2 not in flag) and (method is not None):
            ax.plot(x2, sum, alpha=0.5, color='r', label='Spectral Line')

        #Reverse x-axis if asked for
        if reverse_x is True:
            ax.invert_xaxis()

        #init strings
        a     = file.split('_')
        title = a[1] + '(' + a[2] + '/' + a[3] + ')'
        fname = file.split('/')[-1]
        name  = fname.replace('_hes.log', '_ir-plot')

        #Add titles to plot then save
        plt.ylim(bottom=0)
        plt.legend(loc='upper left')
        plt.xlabel(r'Vibrational Frequency $(cm^{-1})$')
        plt.ylabel(r'Infrared Intensity $(km \cdot mol^{-1})$')
        plt.title(title, fontsize=20)
        plt.tight_layout()
        if savedir is None:
            plt.show()
        else:
            plt.savefig(savedir + name + png)

    #If raman file, make Raman vs Vib Freq plot
    if '_raman' in file:
        ra = data.raman
        vf = hessian(file)[0]

        #init plot plane
        fig, ax = plt.subplots()

        #get x max
        x_max = 0
        for key in vf:
            x = np.float_(vf[key])
            if max(x) > x_max:
                x_max = max(x)

        #iterate through data making lines
        i = 0
        x2   = np.arange(0, x_max*1.25, 0.01)
        sum  = np.zeros(len(x2))
        for key in vf:
            x = np.float_(vf[key])
            y = np.float_(ra[key])

            #check flag, plot v lines
            if 1 not in flag:
                ax.vlines(x, 0, y, label=key, colors=cmap[i])
            if (1 in flag) and (3 not in flag):
                ax.vlines(0, 0, 0, label=key, linestyle='--', colors=cmap[i])

            #Make Gaussian line broadening
            if method == 'Gaussian':
                for a,b in zip(x,y):
                    gfit = gaussian(x2, a, sig, b)
                    sum += gfit

                    #Check flag, plot fits
                    if 3 not in flag:
                        ax.plot(x2, gfit, linestyle='--', color=cmap[i])

            #Make Lorentzian line broadening
            if method == 'Lorentzian':
                for a,b in zip(x,y):
                    lfit = lorentzian(x2, a, sig, b)
                    sum += lfit

                    #Check flag, plot fits
                    if 3 not in flag:
                        ax.plot(x2, lfit, linestyle='--', color=cmap[i])

            i += 1

        #check flag, plot fit sum
        if (2 not in flag) and (method is not None):
            ax.plot(x2, sum, alpha=0.5, color='r', label='Spectral Line')

        #Reverse x-axis if asked for
        if reverse_x is True:
            ax.invert_xaxis()

        #init strings
        a     = file.split('_')
        title = a[1] + '(' + a[2] + '/' + a[3] + ')'
        fname = file.split('/')[-1]
        name  = fname.replace('_raman.log', '_raman-plot')

        #Add titles to plot then save
        plt.ylim(bottom=0)
        plt.legend(loc='upper left')
        plt.xlabel(r'Vibrational Frequency $(cm^{-1})$')
        plt.ylabel(r'Raman Intensity $(\AA^{4}\cdot mol^{-1})$')
        plt.title(title, fontsize=20)
        plt.tight_layout()
        if savedir is None:
            plt.show()
        else:
            plt.savefig(savedir + name + png)

    return



import pkg_resources
from .config        import *
from .input_builder import input_builder
from .sort_inps     import sort_inps

def new_project(maindir, csvfile, initial_coords_dict=None,
                title='Project_Name/', make_inputs=False):
    """
    This function creates a new directory tree for a GAMESS project, also makes
    a couple of text files for use with other functions.

    Parameters
    ----------
    maindir: string
        A directory string (including the final `/`) that points to the
        directory that the project tree will be spawned in.
    csvfile: string
        A directory string (including the final `.csv`) that points to the
        text file containing project information. Read module documentation
        for csv file format.
    initial_coords_dict: dictionary [Optional]
        This should be a dictionary with the key being the specie and the
        value being a list that of its inital coordinates.
    title: string [Optional]
        A directory string (including the final `/`) that will be used as
        the head of project directory tree.
    make_inputs: boolean True/False [Optional]
        if True then new_project will call input_builder at the end.

    Notes 1
    ----------
    If the molecules you wish to build are not already defined in the
    general autogamess coordinate dictionary, then initial_coords_dict
    must be passed.

    To see the autogamess coordianate dictionary simply print out
    >>> ag.dictionaries.molecule_dictionary

    Returns
    ----------
    This function returns nothing

    Notes 2
    ----------
    The format of the spawned directory tree is as follows:

                                    maindir
                                       |
                                     title
                        -------------------------------------
                        |        |        |        |        |
                      Codes    Inps     Logs  Batch_Files  Spreadsheets
                        |        |        |                     |
                  ---------    Block    -----------      1 file per specie
                  |       |             |    |    |
            Text_Files  Scripts      Fail   Pass  Sorted
                                      |      |        |
                                   -------  Block   1 directory per specie
                                   |     |
                             Unsolved   Solved


    Sections in directory tree labled 'Block' are directory trees with the
    following format:
                      1 directory per run type
                                  |
                      1 directory per specie

    Examples
    ----------
    >>> import autogamess as ag
    >>>
    >>> csvfile = './input.csv'
    >>> maindir = './'
    >>> title   = 'Project Title/'
    >>>
    >>> ag.new_project(maindir, csvfile, title=title)
    >>>
    """
    #Spreadsheet header phrase
    version = (' AutoGAMESS Version ' +
                str(pkg_resources.require("autogamess")[0].version) )
    author  = '     by Brian C. Ferrari'

    #Defining directory names
    unsolved = maindir + title + 'Logs/Fail/Unsolved/'
    solved   = maindir + title + 'Logs/Fail/Solved/'
    xldir    = maindir + title + 'Spreadsheets/'
    inputs   = maindir + title + 'Inps/'
    goodlogs = maindir + title + 'Logs/Pass/'
    sorrted  = maindir + title + 'Logs/Sorted/'

    #Define random commands
    fin      = '\nBREAK\n'
    engine   = 'xlsxwriter'
    xlsx     = '.xlsx'

    #Make directories
    try:
        os.makedirs(unsolved)
        os.makedirs(solved)
        os.makedirs(xldir)
    except:
        sys.exit("Project Directory or sub-directories already exist")

    #Read in csv file or Pandas DataFrame
    df = check_data_type(csvfile)

    #Make lists of species, run-types, basis_sets, theories
    runtyps    = [str(x) + '/' for x in list(df['Run Types'].dropna())]
    species    = [str(x) + '/' for x in list(df['Species'].dropna())]
    theories   = list(df['Theory'].dropna())
    cmethods   = list(df['Composite Methods'].dropna())
    basis_sets = list(df['Basis Sets'].dropna()) + list(
                 df['External Basis Sets'].dropna())

    #Make Block directory trees
    for runtyp in runtyps:
        for specie in species:
            os.makedirs(inputs+runtyp+specie)
            os.makedirs(goodlogs+runtyp+specie)

    #Make data for Spreadsheets
    theo = []
    for theory in theories:
        temp    = [theory]*len(basis_sets)
        theo   += temp + ['\n', '\n','\n']

    bs = (basis_sets + ['\n', '\n','\n']) *len(theories)

    #Make dataframe with basis sets names only
    data = {'Theory': theo, 'Basis Set': bs}
    df2  = pd.DataFrame(data=data)

    #Make data for Composite method Sheets
    data = {'Method': cmethods}
    df3  = pd.DataFrame(data=data)


    #More directory making and Excell workbook and sheet making
    for specie in species:
        os.makedirs(sorrted+specie)

        #Define header for Spreadsheets
        header = [version, author, '',
                  'Project Name : ' +  title.replace('/', ''),
                  'Molecule Name: ' + specie.replace('/', '')]

        #Define Excell filename
        xlfilename = xldir + specie.replace('/', xlsx)

        #Initialize writer
        writer = pd.ExcelWriter(xlfilename, engine=engine)

        #Make Sheets and put headers on them all
        for runtyp in runtyps:

            #Define sheet name and make it
            if runtyp == 'Composite/':
                sheet     = runtyp.replace('/', '')
                df3.to_excel(writer, startrow=6, startcol=0, sheet_name=sheet)
                worksheet = writer.sheets[sheet]
            else:
                sheet     = runtyp.replace('/', '')
                df2.to_excel(writer, startrow=6, startcol=0, sheet_name=sheet)
                worksheet = writer.sheets[sheet]

            #Write Header
            for line in header:
                i = header.index(line)
                worksheet.write(i, 0, line)

            #Write Units in header
            u  = 'Units:'
            bl = 'Bond Length (angstroms)'
            ba = 'Bond Angle (degrees)'
            vf = 'Vibrational Frequency (cm^-1)'
            ii = 'Infrared Intensity (Debye^2 angstrom^-2 amu^-1)'
            ra = 'Raman Activity (angstrom^4 amu^-1)'
            fv = 'VSCF Frequency (cm^-1)'
            iv = 'VSCF IR (km mol^-1)'
            hf = 'Heat of Formation (kcal mol^-1)'
            sb = '          '
            if runtyp == 'Optimization/':
                worksheet.write(2, 0, u + sb + bl + sb + ba)
            if runtyp == 'Hessian/':
                worksheet.write(2, 0, u + sb + vf + sb + ii)
            if runtyp == 'Raman/':
                worksheet.write(2, 0, u + sb + ra)
            if runtyp == 'VSCF/':
                worksheet.write(2, 0, u + sb + fv + sb + iv)
            if runtyp == 'Composite/':
                worksheet.write(2, 0, u + sb + hf)

        #Save Excell file
        writer.save()


    #Run Input Builder function
    if make_inputs is True:
        save_dir = inputs
        input_builder(csvfile, save_dir, initial_coords_dict,
                     proj_title = title.replace('/', '\n'))

        #Sorts the freshly made inputs
        projdir = maindir + title
        inpsdir = save_dir
        sort_inps(projdir, inpsdir)

    return



import xlrd as xl
from .config   import *
from .data_finder import *
from .get_data import get_data
from openpyxl import load_workbook

def fill_spreadsheets(projdir=False, sorteddir=False, sheetsdir=False):
    """
    This function fills in the spreadsheets initially generated by new_project
    with data collected from the outfiles of calculations.

    Parameters
    ----------
    projdir: string [Optional]
        This should be a full directory string pointing to the project
        directory initlly created by new_project.
    sorteddir: string [Optional]
        This should be a full directory string pointing to the sorted log
        files directory.
    sheetsdir: string [Optional]
        This should be a full directory string pointing to the spreadsheets
        directory.

    Notes
    ----------
    If projdir is not passed to fill_spreadsheets function then both other
    parameters MUST be passed to it. Similarly if projdir is passed the
    other two parameters MUST be left blank.

    Once `fill_spreadsheets` has parsed the data file, it will move the log file
    into the `Pass` directory implying that the calculation successfully
    terminated, or it will move it to `Fail` directory if termination
    unsuccessful.

    Returns
    -------
    This function returns nothing.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> projdir = './Your Project Title/'
    >>>
    >>> ag.fill_spreadsheets(projdir)
    >>>

    >>> import autogamess as ag
    >>>
    >>> sorteddir = './project/Logs/Sorted/'
    >>> sheetsdir = './project/Spreadsheets/'
    >>>
    >>> ag.fill_spreadsheets(sorteddir=sorteddir, sheetsdir=sheetsdir)
    >>>
    """
    #Defining directory names
    if projdir is not False:
        sorteddir      = projdir + 'Logs/Sorted/'
        sheetsdir      = projdir + 'Spreadsheets/'
        faildir        = projdir + 'Logs/Fail/Unsolved/'
        passdir        = projdir + 'Logs/Pass/'
    else:
        projdir        = '/'.join(sheetsdir.split('/')[0:-2]) + '/'
        faildir        = projdir + 'Logs/Fail/Unsolved/'
        passdir        = projdir + 'Logs/Pass/'

    #Check to make sure sorteddir and sheetsdir exist
    if not os.path.isdir(sorteddir):
        print(error_head + "sorteddir does not exist" + error_tail)
        return
    if not os.path.isdir(sheetsdir):
        print(error_head + "sheetsdir does not exist" + error_tail)
        return

    #Define Sheet names
    opt = 'Optimization'
    hes = 'Hessian'
    ram = 'Raman'
    vsc = 'VSCF'
    cmp = 'Composite'

    #Define Data Column names
    rt = 'Run Time'
    bs = 'Basis Set'
    cp = 'CPU Percentage'
    te = 'Theory'
    me = 'Method'


    for dir in os.listdir(sorteddir):
        df = pd.read_excel(sheetsdir + dir + '.xlsx', index_col=0,
                           sheet_name=None, header=6)

        #Define Correction Factor for Unit Conversion of IR intensity
        wkbk = xl.open_workbook(sheetsdir + dir + '.xlsx')
        wkst = wkbk.sheet_by_name(hes)
        old  = 'Debye^2 angstrom^-2 amu^-1'
        new  = wkst.cell(2, 0).value.split('(')[-1][0:-1].replace('⁻¹', '^-1')

        if old == new:
            irc = 1
        else:
            irc = conversion_factor(old, new)

        for file in os.listdir(sorteddir + dir):
            if '.log' not in file:
                continue
            filename = sorteddir + dir + '/' + file
            theo = file.split('_')[2]
            bset = file.split('_')[3]
            data = get_data(filename)

#---------------OPTIMIZATION FILES-------------------------------------
            if '_opt' in file:
                temp = df[opt]

                if rt not in df[opt]:
                    df[opt][rt] = np.nan
                if cp not in df[opt]:
                    df[opt][cp] = np.nan

                if optimization(filename) == (0,0,0):
                    move2 = faildir + file
                    os.rename(filename, move2)
                    continue

                temp=temp.loc[temp[te]==theo]
                temp=temp.loc[temp[bs]==bset]

                lengths = data.bond_lengths
                for l in lengths:
                    if l not in df[opt]:
                        df[opt][l] = np.nan
                    temp[l] = lengths[l]
                    df[opt].update(temp)

                angles = data.bond_angles
                for a in angles:
                    if a not in df[opt]:
                        df[opt][a] = np.nan
                    temp[a] = angles[a]
                    df[opt].update(temp)

                temp[rt] = data.time
                temp[cp] = data.cpu

                df[opt].update(temp)

                move2 = passdir + opt + '/' + dir + '/' + file
                os.rename(filename, move2)

#-------------------HESSIAN FILES----------------------------------------
            if '_hes' in file:
                temp = df[hes]

                if rt not in df[hes]:
                    df[hes][rt] = np.nan
                if cp not in df[hes]:
                    df[hes][cp] = np.nan

                if hessian(filename) == (0,0,0):
                    move2 = faildir +  file
                    os.rename(filename, move2)
                    continue

                temp=temp.loc[temp[te]==theo]
                temp=temp.loc[temp[bs]==bset]

                frequency = data.vib_freq
                for key in frequency:
                    for i in range(len(frequency[key])):
                        vi = '(' + str(key) + ')Vibrational Frequency ' + str(i)
                        if vi not in df[hes]:
                            df[hes][vi] = np.nan
                        temp[vi] = frequency[key][i]
                        df[hes].update(temp)

                ir = data.ir_inten
                for key in ir:
                    for i in range(len(ir[key])):
                        vi = '(' + str(key) + ')Infrared Intensity ' + str(i)
                        if vi not in df[hes]:
                            df[hes][vi] = np.nan
                        temp[vi] = float(ir[key][i]) * irc
                        df[hes].update(temp)

                temp[rt] = data.time
                temp[cp] = data.cpu

                df[hes].update(temp)

                move2 = passdir + hes + '/' + dir + '/' + file
                os.rename(filename, move2)

#---------------RAMAN FILES------------------------------------------------
            if '_raman' in file:
                temp = df[ram]

                if rt not in df[ram]:
                    df[ram][rt] = np.nan
                if cp not in df[ram]:
                    df[ram][cp] = np.nan

                if raman(filename) == (0,0,0):
                    move2 = faildir + file
                    os.rename(filename, move2)
                    continue

                temp=temp.loc[temp[te]==theo]
                temp=temp.loc[temp[bs]==bset]

                ramans = data.raman
                for key in ramans:
                    for i in range(len(ramans[key])):
                        vi = '(' + str(key) + ')Raman Activity ' + str(i)
                        if vi not in df[ram]:
                            df[ram][vi] = np.nan
                        temp[vi] = ramans[key][i]
                        df[ram].update(temp)

                temp[rt] = data.time
                temp[cp] = data.cpu

                df[ram].update(temp)

                move2 = passdir + ram + '/' + dir + '/' + file
                os.rename(filename, move2)

#-------------------------VSCF Files-------------------------------------
            if '_vscf' in file:
                temp = df[vsc]

                if rt not in df[vsc]:
                    df[vsc][rt] = np.nan
                if cp not in df[vsc]:
                    df[vsc][cp] = np.nan

                if vscf(filename) == (0,0,0):
                    move2 = faildir + file
                    os.rename(filename, move2)
                    continue

                temp=temp.loc[temp[te]==theo]
                temp=temp.loc[temp[bs]==bset]

                vscf_freq = data.vscf_freq
                for key in vscf_freq:
                    vi = '(' + str(key) + ')VSCF Frequency '
                    if vi not in df[vsc]:
                        df[vsc][vi] = np.nan
                    temp[vi] = vscf_freq[key]
                    df[vsc].update(temp)

                vscf_ir = data.vscf_ir
                for key in vscf_ir:
                    vi = '(' + str(key) + ')VSCF IR '
                    if vi not in df[vsc]:
                        df[vsc][vi] = np.nan
                    temp[vi] = vscf_ir[key]
                    df[vsc].update(temp)

                temp[rt] = data.time
                temp[cp] = data.cpu

                df[vsc].update(temp)

                move2 = passdir + vsc + '/' + dir + '/' + file
                os.rename(filename, move2)

#----------------COMPOSITE METHOD Files-------------------------------------
            if '_comp' in file:
                temp = df[cmp]

                if rt not in df[cmp]:
                    df[cmp][rt] = np.nan
                if cp not in df[cmp]:
                    df[cmp][cp] = np.nan

                if composite(filename) == (0,0,0):
                    move2 = faildir + file
                    os.rename(filename, move2)
                    continue

                temp=temp.loc[temp[me]==theo]

                vi = 'Heat of Formation (0K)'
                if vi not in df[cmp]:
                    df[cmp][vi] = np.nan
                temp[vi] = data.hf_0k
                df[cmp].update(temp)

                vi = 'Heat of Formation (298K)'
                if vi not in df[cmp]:
                    df[cmp][vi] = np.nan
                temp[vi] = data.hf_298k
                df[cmp].update(temp)

                temp[rt] = data.time
                temp[cp] = data.cpu

                df[cmp].update(temp)

                move2 = passdir + cmp + '/' + dir + '/' + file
                os.rename(filename, move2)

#---------------------------------------------------------------------------
        #Write Spreadsheets
        book = load_workbook(sheetsdir + dir + '.xlsx')
        with pd.ExcelWriter(sheetsdir + dir + '.xlsx', engine='openpyxl') as writer:
            writer.book = book
            writer.sheets = dict((ws.title, ws) for ws in book.worksheets)
            if opt in df:
                df[opt].to_excel(writer, sheet_name=opt, startrow=6)
            if hes in df:
                df[hes].to_excel(writer, sheet_name=hes, startrow=6)
            if ram in df:
                df[ram].to_excel(writer, sheet_name=ram, startrow=6)
            if vsc in df:
                df[vsc].to_excel(writer, sheet_name=vsc, startrow=6)
            if cmp in df:
                df[cmp].to_excel(writer, sheet_name=cmp, startrow=6)

    return



"""
This file houses the functions that grab data from log files.

Note, the optimization function only grabs raw data
"""
from scipy.spatial import distance
from .config import *

def comp(filename):
    #Read in contents of log file
    log = read_file(filename)

    #Get end of log file, for finding time and cpu
    e   = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end = ctr_f(e, log)

    #Checks is ctr_f fucntion actually found something
    if end != -1:
        i    = ctr_f_last('TOTAL WALL CLOCK TIME', log)
        time = log[i].split()[4]
        cpu  = log[i].split()[9]
    else:
        time = 'Error'
        cpu  = 'Error'

    return [cpu, time]


#---------------------------------------------------------------------
#                        OPTIMIZATION FUNCTION
#---------------------------------------------------------------------

def optimization(filename):
    """
    This function collects data from GAMESS(us) optimization log files.

    Parameters
    ----------
    filename: string
        This should be a string that points to the log file of a
        GAMESS(us) optimization calculation. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    The return is a list, where item 0 is `lengths` and item 1 is `angles`

    lengths: dictionary
        This is a dictionary where the key is in the format `atom1-atom2`
        and the value is a string with the bond length in angstrums.
    angles: dictionary
        This is a dictionary where the key is in the format `atom1-atom2-atom3`
        where atom2 is the central atom and the value is a string with the
        bond angle between the three atoms in radians.

    Notes
    -------
    This function is primarily intended for interal use by AutoGAMESS.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> filename = './AGv0-0-6_NH3_CCSD-T_CC6_opt.log'
    >>>
    >>> ag.data_finder.optimization(filename)
    >>>
    """
    #Read in contents of log file
    log = read_file(filename)

    #Grabs 'Equil' phrase index
    efind = '***** EQUILIBRIUM GEOMETRY LOCATED *****'
    equil = ctr_f(efind, log)

    #Grabs optimized geometries tail index
    hfind = 'INTERNUCLEAR DISTANCES (ANGS.)'
    lhead = len(log) - ctr_f(hfind, log[::-1]) + 2

    #Get end of log file, for finding time and cpu
    e   = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end = ctr_f(e, log)

    #Checks to make sure head and tail exist
    if check_if_exists(filename, equil, ctr_f(hfind, log[::-1]), end):
        return (0,0,0)

    #Makes smaller list to ctr_f through
    temp  = log[equil : lhead]

    #Finds start of full coordinate analysis
    s     = 'COORDINATES OF ALL ATOMS ARE (ANGS)'
    start = ctr_f(s, temp) + 3

    #Make matrix of atom coordinates
    matrix = {}
    i      = 2
    for line in temp[start : len(temp)-4]:

        if line.split()[0] in matrix:
            matrix[str(i)+line.split()[0]] = line.split()[2:5]
            i += 1
        else:
            matrix[line.split()[0]] = line.split()[2:5]

    #Make dictionaries
    lengths = {}
    angles  = {}
    for key in matrix:
        for key2 in matrix:
            a1     = make_xzy(matrix[key] )
            a2     = make_xzy(matrix[key2])
            if a1 == a2:
                continue
            length = distance.euclidean(a1, a2)
            lengths[key + '-' + key2 + ' Bond Length'] = str(length)
            for key3 in matrix:
                a3 = make_xzy(matrix[key3])
                if (a3 == a2) or (a3 == a1):
                    continue
                angle  = find_bond_angle(a1, a2, a3) * (360.0 / (2.0 * np.pi))
                label  = key2 + '-' + key + '-' + key3 + ' Bond Angle'
                angles[label]  = str(angle)

    return [lengths, angles]

#---------------------------------------------------------------------
#                           HESSIAN FUNCTION
#---------------------------------------------------------------------

def hessian(filename):
    """
    This function collects data from GAMESS(us) Hessian log files.

    Parameters
    ----------
    filename: string
        This should be a string that points to the log file of a
        GAMESS(us) hessian calculation. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    The return is a list, where item 0 is `freq` and item 1 is `infr`

    freq: dictionary
        This is a dictionary where the key is the symmetry and the value is
        the vibrational frequency.
    infr: dictionary
        This is a dictionary where the key is the symmetry and the value is
        infrared intensity.

    Notes
    -------
    This function is primarily intended for interal use by AutoGAMESS.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> filename = './AGv0-0-6_NH3_CCSD-T_CC6_hes.log'
    >>>
    >>> ag.data_finder.hessian(filename)
    >>>
    """
    #Read in contents of log file
    log = read_file(filename)

    #Get end of log file, for finding time and cpu
    efind = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end   = ctr_f(efind, log)

    #Checks if ctr_f fucntion actually found something
    if check_if_exists(filename, end):
        return (0,0,0)

    #Find Imaginary Modes
    ifind  = 'TREATED AS IMAGINARY.'
    iindex = ctr_f(ifind, log)
    if iindex == -1:
        imodes = 0
    else:
        imodes = int(log[iindex].split()[3])

    #Find Modes to ignore
    mfind  = 'ARE TAKEN AS ROTATIONS AND TRANSLATIONS.'
    mindex = ctr_f(mfind, log)
    if mindex == -1:
        modes = 0
    else:
        modes  = int(log[mindex].split()[3])

    freq = ctr_f_all('FREQUENCY:',    log)
    ir   = ctr_f_all('IR INTENSITY:', log)
    sym  = ctr_f_all('SYMMETRY:',     log)

    temp1 = flatten([x.split() for x in freq])
    temp2 = flatten([x.split() for x in sym])
    temp3 = flatten([x.split() for x in ir])

    while 'I' in temp1:
        i           = temp1.index('I')
        del(temp1[i])

    #Intitialize dictionary and store vibrational frequency data in it
    freq = {}
    for a,b in zip(temp1[modes:],temp2[modes:]):
        if b not in freq:
            freq[b] = [a]
        else:
            freq[b] += [a]
    #Store Imaginary frequency data in dictionary
    for a,b in zip(temp1[0:imodes],temp2[0:imodes]):
        a = '-' + a
        if b not in freq:
            freq[b] = [a]
        else:
            freq[b] += [a]

    #Intitialize dictionary and store IR intensity data in it
    infr = {}
    for a,b in zip(temp3[modes:],temp2[modes:]):
        if b not in infr:
            infr[b] = [a]
        else:
            infr[b] += [a]
    #Store Imaginary IR intensity data in dictionary
    for a,b in zip(temp3[0:imodes],temp2[0:imodes]):
        if b not in infr:
            infr[b] = [a]
        else:
            infr[b] += [a]


    return [freq, infr]

#---------------------------------------------------------------------
#                           RAMAN FUNCTION
#---------------------------------------------------------------------

def raman(filename):
    """
    This function collects data from GAMESS(us) raman log files.

    Parameters
    ----------
    filename: string
        This should be a string that points to the log file of a
        GAMESS(us) raman calculation. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    ram: dictionary
        This is a dictionary where the key is the symmetry and the
        value is the raman activity.

    Notes
    -------
    This function is primarily intended for interal use by AutoGAMESS.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> filename = './AGv0-0-6_NH3_CCSD-T_CC6_raman.log'
    >>>
    >>> ag.data_finder.raman(filename)
    >>>
    """
    #Read in contents of log file
    log = read_file(filename)

    #Get end of log file, for finding time and cpu
    efind = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end   = ctr_f(efind, log)

    #Checks is ctr_f fucntion actually found something
    if check_if_exists(filename, end):
        return (0,0,0)

    #Find Modes to ignore
    mfind  = 'ARE TAKEN AS ROTATIONS AND TRANSLATIONS.'
    mindex = ctr_f(mfind, log)
    modes  = int(log[mindex].split()[3])

    #Get head and tail of data
    hfind = 'MODE FREQ(CM**-1)  SYMMETRY  RED. MASS  IR INTENS.'
    dhead = ctr_f(hfind, log) + 1 + modes
    tfind = 'THERMOCHEMISTRY AT T=  298.15 K'
    dtail = ctr_f(tfind, log) - 2

    #Checks is ctr_f fucntion actually found something
    if check_if_exists(filename, ctr_f(hfind, log), ctr_f(tfind, log)):
        return (0,0,0)

    ram   = ctr_f_all('RAMAN ACTIVITY:', log)
    sym  = ctr_f_all('SYMMETRY:',     log)

    temp1 = flatten([x.split() for x in ram])
    temp2 = flatten([x.split() for x in sym])

    while 'I' in temp1:
        i           = temp1.index('I')
        temp1[i-1] *= -1
        del(temp1[i])

    ram = {}
    for a,b in zip(temp1[modes:],temp2[modes:]):
        if b not in ram:
            ram[b] = [a]
        else:
            ram[b] += [a]

    return ram

#---------------------------------------------------------------------
#                           VSCF FUNCTION
#---------------------------------------------------------------------

def vscf(filename):
    """
    This function collects data from GAMESS(us) VSCF log files.

    Parameters
    ----------
    filename: string
        This should be a string that points to the log file of a
        GAMESS(us) VSCF calculation. (FULL DIRECTORY STRING REQUIRED)

    Returns
    -------
    The return is a list, where item 0 is `freq` and item 1 is `infr`

    freq: dictionary
        This is a dictionary where the key is the symmetry and the value is
        the vibrational frequency.
    ir: dictionary
        This is a dictionary where the key is the symmetry and the value is
        infrared intensity.

    Notes
    -------
    This function is primarily intended for interal use by AutoGAMESS.

    Example
    -------
    >>> import autogamess as ag
    >>>
    >>> filename = './AGv0-0-6_NH3_CCSD-T_CC6_vscf.log'
    >>>
    >>> ag.data_finder.vscf(filename)
    >>>
    """
    #Read in contents of log file
    log = read_file(filename)

    #Get end of log file, for finding time and cpu
    efind = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end   = ctr_f(efind, log)

    #Checks is ctr_f fucntion actually found something
    if check_if_exists(filename, end):
        return (0,0,0)

    #Get head and tail of data
    hfind1 = 'FREQUENCY, CM-1  INTENSITY, KM/MOL    EXCITATION'
    dhead1 = ctr_f(hfind1, log) + 1
    hfind2 = 'MODE   FREQUENCY, CM-1  INTENSITY, KM/MOL'
    dhead2 = ctr_f(hfind2, log) + 1
    tfind  = '......FINISHED VIBRATIONAL SCF......'
    dtail  = ctr_f(tfind, log)

    if dhead1 != 0:
        #Make Frequency and Infrared Intensity dictionaries
        freq = {}
        ir   = {}
        for line in log[dhead1:dtail]:
            a = line.split()[2] + ' ' + line.split()[3] + ' ' + line.split()[4]
            b = line.split()[0]
            c = line.split()[1]
            freq[a] = b
            ir[a]   = c

    if dhead2 != 0:
        #Make Frequency and Infrared Intensity dictionaries
        freq = {}
        ir   = {}
        for line in log[dhead2:dtail]:
            a = line.split()[0]
            b = line.split()[1]
            c = line.split()[2]
            freq[a] = b
            ir[a]   = c

    return [freq, ir]


#---------------------------------------------------------------------
#                     COMPOSITE METHOD FUNCTION
#---------------------------------------------------------------------

def composite(filename):
    """
    """
    #Read in contents of log file
    log = read_file(filename)

    #Get end of log file, for confirming successful run
    efind = 'EXECUTION OF GAMESS TERMINATED NORMALLY'
    end   = ctr_f(efind, log)

    #Checks is ctr_f fucntion actually found something
    if check_if_exists(filename, end):
        return (0,0,0)

    #Get index of lines with data from log file
    i0k   = ctr_f('HEATS OF FORMATION   (0K):', log)
    i298k = ctr_f('HEATS OF FORMATION (298K):', log)

    #Get data from log file
    hf0k   =   log[i0k].split(':')[1].strip(' ').split(' ')[0]
    hf298k = log[i298k].split(':')[1].strip(' ').split(' ')[0]

    return [hf0k, hf298k]



"""
This is the autogamess plotting module


Author: Brian C. Ferrari
"""
from ..config                  import *


from .make_kde                 import make_kde
from .make_scatter             import make_scatter
from .prep_data                import prep_data



from ..config import *

def prep_data(sheetsdir, ref_theory_map, compare_to, savedir=False):
    """
    """
    #Define Sheet names
    opt = 'Optimization'
    hes = 'Hessian'
    ram = 'Raman'
    vsc = 'VSCF'

    #Define Data Column names/variables
    rt = 'Run Time'
    bs = 'Basis Set'
    cp = 'CPU Percentage'
    te = 'Theory'
    me = 'Method'
    fe = 'Frequency'
    bl = 'Bond Length'
    ir = 'Infrared'
    ra = 'Raman'

    #Other predefined strings
    png      = '.png'
    xlsx     = '.xlsx'
    engine   = 'xlsxwriter'
    X        = 'X'

    #Initialize plot dictionary
    opt_plot_dict  = {}
    freq_plot_dict = {}
    ir_plot_dict   = {}
    ram_plot_dict  = {}

    #Prep non-varied lists to iterate over
    dicts = [opt_plot_dict, freq_plot_dict, ir_plot_dict, ram_plot_dict]
    filts = [bl, fe, ir, ra]

    for file in os.listdir(sheetsdir):

        #Make DataFrame from the excel file
        opt_df = pd.read_excel(sheetsdir + file, index_col=0,
                               sheet_name=opt, header=6)
        hes_df = pd.read_excel(sheetsdir + file, index_col=0,
                               sheet_name=hes, header=6)
        ram_df = pd.read_excel(sheetsdir + file, index_col=0,
                               sheet_name=ram, header=6)

        #Prep DataFrame list for iterating over
        dfs = [opt_df, hes_df, hes_df, ram_df]

        for i in range(len(dicts)):
            df         =   dfs[i]
            regex      = filts[i]
            dicti      = dicts[i]
            ref_theory = ref_theory_map[regex]

            ref_level_str = ref_theory[0] + '/' + ref_theory[1]

            #Pull the subset of the Data Frame that is
            #reference theory level data
            ref_df = df[         df[te] == ref_theory[0] ]
            ref_df = ref_df[ ref_df[bs] == ref_theory[1] ]
            x_vals = ref_df.filter(regex=regex).values.flatten()


            try:
                new_x = np.append(dicti[ref_level_str], x_vals)
                dicti[ref_level_str]  = new_x
            except:
                dicti[ref_level_str]  = x_vals



            for comp_level in compare_to:
                comp_level_str = comp_level[0] + '/' + comp_level[1]

                y_df   = df[     df[te] == comp_level[0] ]
                y_df   = y_df[ y_df[bs] == comp_level[1] ]
                y_vals = y_df.filter(regex=regex).values.flatten()

                if comp_level_str == ref_level_str:
                    continue

                if len(x_vals) != len(y_vals):
                    y_vals = np.empty(len(x_vals))

                try:
                    new_y = np.append(dicti[comp_level_str], y_vals)
                    dicti[comp_level_str]  = new_y
                except:
                    dicti[comp_level_str]  = y_vals

    if savedir:
        #Define header for Spreadsheets
        header = [version, author, '',
                  'Project Name : To Plot',
                  'Molecule Name:  Bunch of them']

        #Define Excell filename
        xlfilename = savedir + 'To_Plot.xlsx'

        #Initialize writer
        writer = pd.ExcelWriter(xlfilename, engine=engine)

        for i in range(len(dicts)):
            sheet = filts[i]
            data  = pd.DataFrame(dict([ (k,pd.Series(v)) for k,v in dicts[i].items()]))

            data.to_excel(writer, startrow=6, startcol=0, sheet_name=sheet)
            worksheet = writer.sheets[sheet]

            #Write Header
            for line in header:
                i = header.index(line)
                worksheet.write(i, 0, line)

            #Write Units in header
            u  = 'Units:'
            bl = 'Bond Length (Å)'
            vf = 'Vibrational Frequency (cm⁻¹)'
            ii = 'Infrared Intensity (km mol⁻¹)'
            ra = 'Raman Activity (angstrom⁴ amu⁻¹)'
            fv = 'VSCF Frequency (cm⁻¹)'
            iv = 'VSCF IR (km mol⁻¹)'
            hf = 'Heat of Formation (kcal mol⁻¹)'
            sb = '          '
            if sheet == 'Bond Length':
                worksheet.write(2, 0, u + sb + bl)
            if sheet == 'Frequency':
                worksheet.write(2, 0, u + sb + vf)
            if sheet == 'Infrared':
                worksheet.write(2, 0, u + sb + ii)
            if sheet == 'Raman':
                worksheet.write(2, 0, u + sb + ra)

        #Save Excell file
        writer.save()
        return pd.read_excel(xlfilename, index_col=0, header=6)
    else:
        full_data = {}
        for i in range(len(dicts)):
            name  = filts[i]
            data  = pd.DataFrame(dicts[i])

            full_data[name] = data

        return pd.DataFrame(full_data)



import setuptools

setuptools.setup(
    name="autogamess",
    version="1.1.57",
    author="Brian C. Ferrari",
    author_email="brianf1996@knights.ucf.edu",
    description="This is a python module for automating Raman calculations using GAMESS(us).",
    url="https://github.com/Cavenfish/autogamess",
    packages=setuptools.find_packages(),
    install_requires=["numpy", "scipy", "pandas", "basis_set_exchange", "PeriodicElements",
                      "openpyxl", "matplotlib", "xlrd", "xlsxwriter"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows :: Windows 10",
    ],
)



from .config import *

def prep_restart(inpfile, rstfile):
    """

    """
    vib = '$VIB2'

    rst = read_file(rstfile)

    i = ctr_f(rst, vib)

    with open(inpfile, 'a') as f:
        f.write(rst[i:])

    return



from .config import *

def sort_logs(projdir, logsdir):
    """
    This function sorts all the loose log files in the 'Logs' directory.

    Parameters
    ----------
    projdir: string
        A directory string (including the final `/`) that points to the
        project head directory.
    logsdir: string
        A directory string (including the final `/`) that points to the
        directory containing the log files.

    Returns
    ----------
    This function returns nothing

    Notes
    ----------
    For this function to work properly the project directory tree must be
    in the exact format that the 'new_project' fucntion spawned it in.

    Examples
    ----------
    >>>import autogamess as ag
    >>>
    >>>projdir = './Example/'
    >>>logsdir = './logs/'
    >>>
    >>>ag.sort_logs(projdir, logsdir)
    >>>
    """
    #Defining directory names
    sorteddir      = projdir + 'Logs/Sorted/'

    #Checks if sorteddir is real directory
    if not os.path.isdir(sorteddir):
        sorteddir = projdir

    #Defining extension names
    lgext    = '.log'

    for filename in os.listdir(logsdir):

        #Skips non-log files
        if lgext not in filename:
            continue

        #Gets molecule name, then puts string in directory format
        specie  = filename.split('_')[1]
        specie += '/'

        #Define the directory to put this particular file in
        move2 = sorteddir + specie

        #Checks if move2 directory exists, if not then makes it
        if not os.path.isdir(move2):
            os.makedirs(move2)

        #Define `before/after` of rename command to move file
        before = logsdir + filename
        after  = move2   + filename

        #Moves log to proper directory
        os.rename(before, after)


    return
